<?php

namespace App\Repository;

use App\Entity\Course;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Common\Persistence\ManagerRegistry;

/**
 * @method Course|null find($id, $lockMode = null, $lockVersion = null)
 * @method Course|null findOneBy(array $criteria, array $orderBy = null)
 * @method Course[]    findAll()
 * @method Course[]    findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null)
 */
class CourseRepository extends ServiceEntityRepository
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, Course::class);
    }

    /**
    * @param string $role
    *
    * @return array
    */
    public function getAvailableCourses($user_role, $language)
    {
       $qb = $this->_em->createQueryBuilder();
       $qb->select('c')
           ->from('App:Course', 'c')
           ->where('c.roles like :role1')
           ->andWhere('c.state = :status')
           ->andWhere('c.language = :language')
           ->setParameters(array(
            'role1'     => '%"'.$user_role.'"%',
            'status'    => 'state.published',
            'language'    => $language,
           ));

       return $qb->getQuery()->getResult();
    }

    /**
    * @param string $role
    *
    * @return array
    */
    public function getCourseByRequirements($user_id, $requirements)
    {
       $qb = $this->_em->createQueryBuilder();
       $qb->select('c')
           ->from('App:Course', 'c')
           ->leftJoin('c.progress', 'p')
           ->join('p.student', 's')
           ->where('c.requirements = :requirements')
           ->andWhere('c.state = :state')
           ->andWhere('s.id = :user_id')
           ->setParameters(array(
            'user_id'           => $user_id,
            'state'             => 'state.published',
            'requirements'      => $requirements,
           ));


        try {
            return $qb->getQuery()->getOneOrNullResult();
        } catch (\Doctrine\ORM\NoResultException $exception) {
            return null;
        }
    }

    /**
    * @param string $role
    *
    * @return array
    */
    public function getCourseProgressByRequirements($user_id, $requirements)
    {
       $qb = $this->_em->createQueryBuilder();
       $qb->select('c.id, COUNT(p) AS progress_total, MIN(p.createdAt) AS enrolled_date')
           ->from('App:Course', 'c')
           ->leftJoin('c.progress', 'p')
           ->join('p.student', 's')
           ->where('c.requirements = :requirements')
           ->andWhere('c.state = :state')
           ->andWhere('s.id = :user_id')
           ->groupBy('c.id')
           ->setParameters(array(
            'user_id'           => $user_id,
            'state'             => 'state.published',
            'requirements'      => $requirements,
           ));

        $result =  $qb->getQuery()->getResult();

        if($result){
            $current_date   = new \DateTime();
            $init_date      = new \DateTime( $result[0]['enrolled_date'] );
            $final_date     = new \DateTime( $current_date->format('Y-m-d') );

            $interval = date_diff($init_date, $final_date);
            $days_inverval = $interval->format('%a');

            $query = $this->_em->createQueryBuilder();
            $query->select('count(p.id)')
            ->from('App:Principle', 'p')   
            ->join('p.unit', 'u')     
            ->where('u.course = :course')
            ->andWhere('p.state = :state')
            ->setParameters(array(
                'course'         => $result[0]['id'],
                'state'          => "state.published"
                ))
                ;
    
            $principles_total = $query->getQuery()->getSingleScalarResult();            
            $progress_percentaje =  round(($result[0]['progress_total'] * 100) / $principles_total) ;

            $data = [
                'course_id' => $result[0]['id'],
                'enrolled_date' => $result[0]['enrolled_date'],
                'enrolled_days' => $days_inverval,
                'progress_total' => $result[0]['progress_total'],
                'principles_total' => $principles_total,
                'progress_percentaje' => $progress_percentaje
            ];
        }
        else{
            $data = [
                'course_id' => "",
                'enrolled_date' => "",
                'enrolled_days' => 0,
                'progress_total' => 0,
                'principles_total' => 0,
                'progress_percentaje' => 0
            ];
        }
        

        try {
            return $data;
        } catch (\Doctrine\ORM\NoResultException $exception) {
            return null;
        }
    }

    /**
    * @param string $role
    *
    * @return array
    */
    public function getCoursePercentajeProgressByCourse($user_id, $course_id)
    {
       $qb = $this->_em->createQueryBuilder();
       $qb->select('c.id, COUNT(p) AS progress_total, MIN(p.createdAt) AS enrolled_date')
           ->from('App:Course', 'c')
           ->leftJoin('c.progress', 'p')
           ->join('p.student', 's')
           ->where('c.id = :course_id')
           ->andWhere('s.id = :user_id')
           ->groupBy('c.id')
           ->setParameters(array(
            'user_id'           => $user_id,
            'course_id'         => $course_id,
           ));

        $result =  $qb->getQuery()->getResult();

        if($result){
            $query = $this->_em->createQueryBuilder();
            $query->select('count(p.id)')
            ->from('App:Principle', 'p')   
            ->join('p.unit', 'u')     
            ->where('u.course = :course')
            ->setParameters(array(
                'course'         => $result[0]['id'],
                ))
                ;
    
            $principles_total = $query->getQuery()->getSingleScalarResult();            
            $progress_percentaje =  round(($result[0]['progress_total'] * 100) / $principles_total) ;

        }
        else{
            $progress_percentaje =  0 ;
        }
        

        try {
            return $progress_percentaje;
        } catch (\Doctrine\ORM\NoResultException $exception) {
            return null;
        }
    }

    /**
    * @param string $role
    *
    * @return array
    */
    public function getCourseByRequirementsLanguage($user_id, $requirements, $language)
    {
       $qb = $this->_em->createQueryBuilder();
       $qb->select('c')
           ->from('App:Course', 'c')
           ->leftJoin('c.progress', 'p')
           ->join('p.student', 's')
           ->where('c.requirements = :requirements')
           ->andWhere('c.state = :state')
           ->andWhere('c.language = :language')
           ->andWhere('s.id = :user_id')
           ->setParameters(array(
            'user_id'           => $user_id,
            'state'             => 'state.published',
            'requirements'      => $requirements,
            'language'          => $language
           ));


        try {
            return $qb->getQuery()->getOneOrNullResult();
        } catch (\Doctrine\ORM\NoResultException $exception) {
            return null;
        }
    }
}
