<?php

namespace App\Repository;

use App\Entity\Progress;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Common\Persistence\ManagerRegistry;
use Doctrine\ORM\Tools\Pagination\Paginator;

/**
 * @method Progress|null find($id, $lockMode = null, $lockVersion = null)
 * @method Progress|null findOneBy(array $criteria, array $orderBy = null)
 * @method Progress[]    findAll()
 * @method Progress[]    findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null)
 */
class ProgressRepository extends ServiceEntityRepository
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, Progress::class);
    }

    

    /**
    * @param string $course_id
    *
    * @return array
    */
    public function studentProgressList($course_id){
      
        $query = $this->getEntityManager()
            ->createQuery( 
            'SELECT u, p FROM App:User u
             JOIN u.progress p
             WHERE p.course = :course
             ORDER BY p.createdAt DESC' 
        )->setParameters(array(
          'course' => $course_id,
        ));
  
        try {
            return $query->getResult();
        } catch (\Doctrine\ORM\NoResultException $exception) {
            return null;
        }
    }

    /**
    * @param string $name
    *
    * @return array
    */
    public function studentProgressListPagination($key, $page, $page_size, $order_column, $order_direction, $course_id, $program )
    {
        
        $query = $this->_em->createQueryBuilder('u');
        $query->select('u.id AS id,' .( $program !== 'without_requirements' ? 'CASE WHEN pr.id IS NOT NULL THEN pr.state ELSE 0 END AS status,': ''). ' u.whatsapp AS whatsapp, u.country AS country, g.name AS group,  MIN(p.createdAt) AS enrolled, CONCAT(u.firstName, \' \', u.lastName) AS participant_name,  COUNT(p) AS progress, u')
            ->from('App:User', 'u')
            ->leftJoin('u.progress', 'p')
            ->leftJoin('u.studentgroup', 'sg');
        if($program !== 'without_requirements'){
            $query->leftJoin($program, 'pr', 'WITH', 'u.id = pr.student');
        }      
        $query->join('sg.group', 'g')
            ->where('CONCAT(u.firstName, \' \', u.lastName, \' \', g.name ) LIKE :name')   
            ->andWhere('p.course = :course_id')
            ->groupBy('u.id')
            ->setParameters(array(
                'name'          => '%'.$key.'%', 
                'course_id'     => $course_id
            ))
            ->orderBy($order_column, $order_direction);


        $paginator = new Paginator($query);
        $total_items = $paginator->count();
        $total_pages = (int) ceil($total_items / $page_size);

        $paginator->getQuery()
            ->setFirstResult($page_size * ($page - 1))       
            ->setMaxResults($page_size);
        
        $result = [
            'list'              => $paginator->getIterator(),
            'items_total'       => $total_items,
            'pages_total'       => $total_pages
        ];
       
        return $result;
    }

    /**
    * @param string $name
    *
    * @return array
    */
    public function enrrolledByCourseDate($student_id, $course_id )
    {
        
        $query = $this->_em->createQueryBuilder('u');
        $query->select('u.id AS id, MIN(p.createdAt) AS enrolled_date,  COUNT(p) AS progress, u')
            ->from('App:User', 'u')
            ->leftJoin('u.progress', 'p')
            ->where('p.course = :course_id')   
            ->andWhere('p.course = :course_id')
            ->groupBy('u.id')
            ->setParameters(array(
                'student_id'    => $student_id,
                'course_id'     => $course_id
            ));


            return $query->getQuery()->getResult();
    }

    /**
    * @param string $course_id
    *
    * @return array
    */
    public function studentPrinciples($course_id, $student_id){
      
       $qb = $this->_em->createQueryBuilder();
       $qb->select('count(p.id)')
           ->from('App:Progress', 'p')
           ->where('p.course = :course')
           ->andWhere('p.student = :student')
           ->setParameters(array(
            'course'         => $course_id,
            'student'        => $student_id,
           ))
           ;

       return $qb->getQuery()->getSingleScalarResult();
    }

    /**
    * @param string $course_id
    *
    * @return array
    */
    public function answerPendingCorrections($student_id){
      
        $qb = $this->_em->createQueryBuilder();
        $qb->select('p')
            ->from('App:Progress', 'p')
            ->andWhere('p.student = :student')
            ->andWhere('p.state = :state')
            ->groupBy('p.course')
            ->setParameters(array(
             'student'        => $student_id,
             'state'          => "state.correction"
            ))
            ;
 
        return $qb->getQuery()->getResult();
     }

    /**
    * @param string $student_id
    *
    * @return array
    */
    public function deleteProgressByStudent($course_id, $student_id){
      
        $qb = $this->_em->createQueryBuilder();
        $qb->from('App:Progress', 'p')
            ->where('p.course = :course')
            ->andWhere('p.student = :student')
            ->setParameters(array(
             'course'         => $course_id,
             'student'        => $student_id,
            ));
 
        return $qb->getQuery()->execute();
     }

     /**
    * @param string $course_id
    *
    * @return array
    */
    public function deleteProgressByCourse($course_id, $student_id)
    {
       $qb = $this->_em->createQueryBuilder();
       $qb->delete('App:Progress', 'p')
            ->where('p.course = :course')
            ->andWhere('p.student = :student')
           ->setParameters(array(
            'course'         => $course_id,
            'student'        => $student_id,
           ));

       return $qb->getQuery()->getResult();
    }

  

    /**
    * @param string $student_id
    *
    * @return array
    */
    public function deleteAnswerCommentsByStudent($course_id, $student_id){
      
        $qb = $this->_em->createQueryBuilder();
        $qb->update('App:Progress', 'p')
            ->set('p.state', ':state')
            ->where('p.course = :course')
            ->andWhere('p.student = :student')
            ->setParameters(array(
             'course'         => $course_id,
             'student'        => $student_id,
             'state'          => 'state.approved'
            ));
 
        return $qb->getQuery()->execute();
     }
}
